"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
var React = require("react");
var classNames = require("classnames");
var data = require("../../data");
var utils_1 = require("../../utils");
var FormInput_1 = require("../FormInput");
var NestedFormInput = (function (_super) {
    __extends(NestedFormInput, _super);
    function NestedFormInput(props) {
        var _this = _super.call(this, props) || this;
        _this.renderEntry = _this.renderEntry.bind(_this);
        _this.onCreateEntry = _this.onCreateEntry.bind(_this);
        _this.onDeleteEntry = _this.onDeleteEntry.bind(_this);
        _this.onEntryValueChanged = _this.onEntryValueChanged.bind(_this);
        _this.fieldStatus = {
            error: '',
            nestedStatus: [],
        };
        return _this;
    }
    NestedFormInput.prototype.render = function () {
        return (React.createElement("div", { className: 'form-input-nested' },
            React.createElement("div", { className: 'form-input-nested-label' }, this.props.field.label),
            React.createElement("div", { className: 'form-input-nested-content' },
                this.props.value.map(this.renderEntry),
                this.renderCreateButton())));
    };
    NestedFormInput.prototype.renderEntry = function (entry, index) {
        var showDeleteButton = this.props.value.length !== 1;
        return (React.createElement(NestedFormEntry, { key: entry.id, index: index, value: entry, context: this.props.context, field: this.props.field, registry: this.props.registry, showDeleteButton: showDeleteButton, onChange: this.onEntryValueChanged, onDelete: this.onDeleteEntry, attempt: this.props.attempt, nestedFormEntryWrapper: this.props.nestedFormEntryWrapper }));
    };
    NestedFormInput.prototype.onCreateEntry = function (event) {
        event.stopPropagation();
        event.preventDefault();
        var entries = this.props.value.slice();
        entries.push({ id: utils_1.generateID() });
        this.props.onValueChange(this.props.field, entries, this.fieldStatus);
    };
    NestedFormInput.prototype.onDeleteEntry = function (index) {
        var _this = this;
        if (this.props.onBeforeDeleteEntry) {
            var before = this.props.onBeforeDeleteEntry(this.props.value[index]);
            if (before && before.then) {
                before.then(function (result) {
                    if (result) {
                        _this.deleteEntry(index);
                    }
                });
            }
            else {
                console.warn('Must return a promise');
            }
        }
        else {
            this.deleteEntry(index);
        }
    };
    NestedFormInput.prototype.deleteEntry = function (index) {
        var entries = this.props.value.slice();
        var deletedEntry = this.props.value[index];
        entries.splice(index, 1);
        delete this.fieldStatus.nestedStatus[deletedEntry.id];
        this.updateFieldStatus();
        this.props.onValueChange(this.props.field, entries, this.fieldStatus);
    };
    NestedFormInput.prototype.onEntryValueChanged = function (value, formStatus, index) {
        var newValue = this.props.value.slice();
        newValue[index] = value;
        this.fieldStatus.nestedStatus[value.id] = formStatus;
        this.updateFieldStatus();
        this.props.onValueChange(this.props.field, newValue, this.fieldStatus);
    };
    NestedFormInput.prototype.updateFieldStatus = function () {
        var error = false;
        for (var id in this.fieldStatus.nestedStatus) {
            if (data.isFormError(this.fieldStatus.nestedStatus[id])) {
                error = true;
            }
        }
        this.fieldStatus.error = error ? 'nestedError' : '';
    };
    NestedFormInput.prototype.renderCreateButton = function () {
        var source = this.props.createButton;
        if (!source || typeof source === 'string') {
            return (React.createElement("button", { className: 'form-input-nested-button form-input-nested-create-button', type: 'button', onClick: this.onCreateEntry }, source || 'Create'));
        }
        if (React.isValidElement(source)) {
            return React.cloneElement(source, {
                onClick: this.onCreateEntry,
                className: classNames(source.props.className, 'form-input-nested-button form-input-nested-create-button'),
            });
        }
        throw new Error('Cannot render create button');
    };
    return NestedFormInput;
}(React.PureComponent));
NestedFormInput.defaultProps = {
    value: [{ id: utils_1.generateID() }],
    createButton: 'Create',
};
exports.NestedFormInput = NestedFormInput;
var NestedFormEntry = (function (_super) {
    __extends(NestedFormEntry, _super);
    function NestedFormEntry() {
        var _this = _super.call(this) || this;
        _this.onValueChanged = _this.onValueChanged.bind(_this);
        _this.onDeleted = _this.onDeleted.bind(_this);
        return _this;
    }
    NestedFormEntry.prototype.render = function () {
        var formInput = React.createElement(FormInput_1.FormInput, { fields: this.props.field.fields, registry: this.props.registry, value: this.props.value, context: this.props.context, onChange: this.onValueChanged, attempt: this.props.attempt });
        if (this.props.nestedFormEntryWrapper) {
            return React.createElement(this.props.nestedFormEntryWrapper, {
                index: this.props.index,
                field: this.props.field,
                showDeleteButton: this.props.showDeleteButton,
                onDelete: this.onDeleted,
            }, formInput);
        }
        else {
            var deleteBtn = React.createElement("button", { type: 'button', className: 'form-input-nested-entry-delete', onClick: this.onDeleted }, "Delete");
            return (React.createElement("div", { className: 'form-input-nested-entry' },
                this.props.showDeleteButton && deleteBtn,
                formInput));
        }
    };
    NestedFormEntry.prototype.onDeleted = function () {
        this.props.onDelete(this.props.index);
    };
    NestedFormEntry.prototype.onValueChanged = function (value, formStatus) {
        this.props.onChange(value, formStatus, this.props.index);
    };
    return NestedFormEntry;
}(React.PureComponent));
